# Introduction to the Smart Device Template

## Table of Contents
1. [Introduction](#Introduction)
2. [Definitions](#Definitions)
3. [How should SDT work?](#Work)
4. [Related aspects which are out of scope of the SDT](#related)

<a name="Introduction"/></a>
## Introduction

The Smart Device Template (SDT) is an initiative from [oneM2M](http://www.onem2m.org) to find consensus amongst various SDOs and industry alliances to derive a common approach for device modeling. oneM2M and partners have the approach to agree on a set of automation functionalities, following a common syntax, which are sufficient to model most device functions.

Originally initiated by the [Home Gateway Initiative (HGI)](http://www.homegatewayinitiative.org), oneM2M is now the owner and maintainer of the SDT.

At the time of writing, every software developed for Internet-of-Things gateways and applications, for example gateways for home or building automation, needs to be capable of using various different protocols (ZigBee, ZWave, KNX, OPC UA, EchonetLite, DECT ULE, etc) to interact with a range of devices designed for the IoT environment. This adds extreme overheads in integrating, testing and updating code. The purpose of the SDT is to describe devices and device services in a way which is independent of the LAN technology and protocols, in a format which is convenient and reliable for integration in modern code (C/C++, Go, Java, Javascript, Python, ...).

### Goals
The key goals of the SDT are: 

1. keep it simple, especially for manufacturers to contribute device information;
2. enable and support modularity for functions and device types;
3. make it easy for developers to create unified APIs;
4. be independent of underlying LAN technologies and protocols;
5. enable extendibility of the system in place without service interruption;
6. allow a pass-through mechanism to enable use of proprietary or technology-specific functions.

In general a description of device (or complex appliance) behaviour can be made in many ways, with various kinds of constraints:

1. no constraints (e.g. using OWL 2.0 or RDF)
2. moderate constraints (e.g. using XML and a related extensible XSD template)
3. strict constraints (typical for a device certified to interoperate with a specific LAN protocol)

oneM2M chose to use the approach "moderate constraints" (XSD based) because for software development it offers ease of use and a good compromise. In particular, if there are few or no constraints on control parameters then few automatic checks can be made to detect if the software parameters are appropriate for each device integrated. XML and XSD languages were chosen because they are familiar to many developers, can be parsed and validated with common software tools, and can still be created and interpreted by humans if necessary. SDT intended use is for modeling, not as a presentation layer protocol, therefore the size of individual templates is not important.

### Guidelines for Structured Information Models

oneM2M believes that Device information models based on XML and extensible XSD need some guidelines. If every possible feature of every existing LAN technology and appliance were allowed to be described in any formally correct way, then the results would be a modern Babel, no better than today's system of widely different and wildly competing automation protocols. 

Therefore oneM2M proposes to recommend a certain structure (or template) for the information model(s), but to allow extensions. Naturally, the more industry consensus is achieved for a single recommended template, the greater the utility for software developers (and in the end users and customers).

The SDT approach is to define re-usable basic functions (or services), labelled "[ModuleClass](SDT_Components.md#ModuleClass)" in the figure below, which can represent the typical functions found in many IoT systems, such as "on/off", "dim a lamp", "receive events from sensor", "read data from sensor", etc. Each ModuleClass is composed of a (preferable small) number of actions, datapoint read/write operations, or asynchronous events. For example, an "on/off" ModuleClass would consist perhaps of just one [Action](SDT_Components.md#Action), but a "ReadKeypad" Action might have a number of possible events, each with some data value and (usually) a sequence-ID or timestamp start/stop to indicate when and how long each key was pressed.

![SmartHome Device Template for a generic device (UML, basic entities)](images/SDT_simplified.png)  
**SmartHome Device Template for a generic device (UML, basic entities)**

The SDT represents the device models introduced in the above figure by using an XSD schema to allow formal checking of compliance for XML device descriptions of specific appliances. The modularity  goal in the schema is achieved with re-usable XML fragments (for example the "ModuleClass" in the figure). 

Complex [Devices](SDT_Components.md#Devices), appliances or even [Products](SDT_Components.md#Products) can then be described by an appropriate set or collection of the agreed [ModuleClasses](SDT_Components.md#ModuleClass), as indicated in the figure, which also shows an optional "[Property](SDT_Components.md#Property)" fragment to allow noting of static information such as device manufacturer name, device firmware version, etc.

oneM2M has discussed with many SDOs to validate these concepts. SDT is designed to take into account the list of "services" compiled by the [SAREF project](https://sites.google.com/site/smartappliancesproject/home).

The SDT supports the use of a set of templates for generic devices or appliances (e.g. for a *dimmable lamp*, a basic *washing machine*, etc), which would be specific instances of the "[DeviceClass](SDT_Components.md#DeviceClass)", which form the basis of APIs used by application developers. These templates can also be referenced by manufacturers creating XML documents to describe their specific Products. For example, the SDT enables specification of a generic washing machine template, with *on/off*, *set-wash-temperature*, *pause* and a few other commands, which could be referenced and extended by a manufacturer as the schema for an XML description of a specific washing machine product. The SDT allows for vendor-specific additional commands (ModuleClasses) to suit specific product types. 

The interoperability benefits can potentially partially be obtained even without a fully complete interoperability of the SDT. For example, the most common functions can be modeled with SDT, and more particular functions can be modeled with technology-specific, proprietary, or seldom-used aspects. 

Various details about the recommended structure for SDTs are described in the next sections. The key point to keep in mind is that oneM2M sought a compromise between, at the one extreme, complete flexibility (which could describe any device, of any complexity) and, at the other extreme, a rigid structure which could be 100% validated and lead to validated software APIs.

<a name="Definitions"/></a>
## Definitions

This section provides an overview about high-level SDT 4.0 definitions and element hierarchy. Terms to be described in detail in this section are:

| Term |  Definition  |
|------|------------|
|[Domain](SDT_Components.md#Domain)  | Unique name, or "wrapper" which acts like a namespace, set by the organization, company, or project creating the template, allowing reference to a package of definitions for the contained ModuleClasses and device definitions. Can be referenced when extending Products, ModuleClasses, and data types. It has two possible uses: to select the scope of a technology domain, or to set the scope of a use case domain (like Home, SmartGrid, etc) |
|[ProductClass](SDT_Components.md#ProductClass)  | A concrete device model with deterministic device Properties and ModuleClasses, without optionality. It is deemed as a specialized implementation of a DeviceClass that can be manufactured. |
|[DeviceClass](SDT_Components.md#DeviceClass)  | A physical, addressable, identifiable appliance, sensor, or actuator. |
|[Sub-Device](SDT_Components.md#SubDevice) | A device (usually one of several) which may be embedded in or attached to a (full) device. It is not designed to be operated as a standalone device. |
| [ModuleClass](SDT_Components.md#ModuleClass) | Specification of a single service with one or more service methods, the involved abstracted data model and related events. The expectation is that each separate service which may be used in many kinds of devices (like *PowerON/OFF*, *Open/Close*, ...) will be described by a ModuleClass which can be re-used in many *DeviceClass* or *ProductClass* definitions. |


A major decision, facilitating validation of code and signaling, was to describe services (functionality) of devices in terms of [ModuleClasses](SDT_Components.md#ModuleClass) made up of combinations of three kinds of elements:

1. [DataPoints](SDT_Components.md#DataPoint) which are aspects of a functionality that can be read and/or written,
2. [Actions](SDT_Components.md#Action) which consist of more complex sequences of operations;
3. [Events](SDT_Components.md#Event) which can be signaled ("published") by devices asynchronously.

Any of these elements is optional.

This [ModuleClass](SDT_Components.md#ModuleClass) structure is a major part of the SDT which is illustrated in detail in the following figure:
 
![UML description of device functionality in terms of DataPoints, Actions, and Events](images/MC.Action.DataPoint.Event.png)  
**UML description of device functionality in terms of DataPoints, Actions, and Events**

<a name="Work"/></a>
## How should SDT work?

The basic concept is that a manufacturer, organisation, service provider or global SDO would define its preferred Smart Device Template, in XML, specified by and based on an XSD. Using that XSD, manufacturers or indeed hobbyists could "describe" existing or new devices by means of XML files, specifying the capabilities and the parameters needed to control the devices.

Assuming that the XML files conform to the specified XSD and to some guidelines described in this document, software developers could readily create APIs able to "parse" the XML-descriptions of devices and (assuming the underlying LAN technology of the device is supported by the software/hardware environment in the gateway) operate the equipment.

The key to making software reliably interoperate with various technologies is to define in the SDT a finite and convenient to use number of functions (which is the decision of the SDO which are commonly used and can therefore be reliably re-used in software for one type of device or another.

For the convenience of users and developers, it would also be possible to collect the device descriptions of common "modules" (types of) appliances so that the operations of "a generic air-conditioner" could be agreed and re-used often, adapting descriptions of special models with just some special features added as local extensions. Agreeing on the definition of "a generic XYZ appliance" is rather time-consuming, so such "repository" may not become standardised, however the basic approach has huge benefits even if such an archive (also known as a "hierarchical ontology") is never formally agreed.

<a name="related"/></a>
## Related aspects which are out of scope of the SDT

The SDT defines the structure of all compliant XML descriptions. Each XML description of a specific device is definable at the time of manufacture of the device and can therefore only contain "static" information: (a) manufacturer data in the form of documentation elements and properties, and (b) device capability information detailing the firmware operations and types/meanings of input/output variables. 

NOT directly part of this work is a related but separate aspect of every (gateway) software development: a "device abstraction layer" (or "service layer") which can translate between (a) software APIs written based on a particular SDT and (b) the "commands" expected by several different LAN protocols and their hardware controllers. 

Programmers developing a "device abstraction layer" for software in a gateway need to create run-time representations of all the recognized devices, their operations and their actual states. This internal "information model" needs to be updated in real time as the devices and the users interact. Programmers may be tempted to use the SDT structure to organize their real-time information model, adding additional information elements for the current state of each device, for some kind of "history" of commands sent/acknowledged, the user etc. 

The above is an efficient approach BUT it must be clear that the real-time state information and history of events, etc, can NOT be represented in a pre-defined SDT and in the XML giving specific details for a device.

---

The detailed documentation about SDT components can be found [here](SDT_Components.md).
