# SDT Components

## Table of Contents

1. [Basic Components](#BasicComponents)
    1. [Domain](#Domain)
    2. [ModuleClass](#ModuleClass)
        1. [DataPoint](#DataPoint)
        2. [Action](#Action)
            1. [Arg](#Arg)
        3. [Event](#Event)
    3. [DeviceClass](#DeviceClass)
        1. [SubDevice](#SubDevice)
    4. [ProductClass](#ProductClass)
    5. [Property](#Property)
2. [Data Types](#DataTypes)
    1. [DataType](#DataType)
        1. [Constraint](#Constraint)
        2. [TypeChoice](#TypeChoice)
    2. [Simple](#Simple)
    3. [Struct](#Struct)
    4. [Array](#Array)
    5. [Enum](#Enum)
        1. [ExtendExclude](#ExtendExclude)
3. [Extending (Inheriting)](#Extending)
    1. [Extend](#Extend)
        1. [Include](#ExtendInclude)
        2. [Exclude](#ExtendExclude)
        3. [ExtendType](#ExtendType)
4. [Documentation (Doc)](#Documentation)

---

## SDT Overview

The following UML diagrams presents an overview of the structure (elements) of every SDT which is conformant with these guidelines. As implied in the above descriptions, there can be many different choices of SDT details, each one optimized for a particular market segment and the types of devices used in that market segment. Obviously an unnecessary proliferation is counter-productive, but as long as each SDT conforms to the structure shown below then it will be possible with little or modest effort for software to be adapted accordingly. 

The key to the diagram elements of the UML diagrams and the snippets in the following sections follow this figure:

![](images/SDT_UML_Key.png)

The syntax used in the diagrams to model an XML Schema Definition (XSD) as an UML diagram follows the following approaches:


- [Design XML schemas using UML](http://www.ibm.com/developerworks/library/x-umlschem/)
- [UML For W3C XML Schema Design](http://www.xml.com/pub/a/2002/08/07/wxs_uml.html)


<a name="BasicComponents"></a>
## Basic Elements
![](images/SDT_UML_Basic_Elements.png)


<a name="Domain"></a>
### Domain

![](images/Domain.png)

The *Domain* element allows labeling of different SDT templates for different technologies and/or industry segments ("verticals"): for example eHealth and Building Management might prefer quite different detailed structures/templates. This also helps keep information in human-friendly and manageable blocks. It is assumed that there will be multiple "SDT Templates", and some of them may be completely proprietary.

It can also be used to collect all specified [ModuleClasses](#ModuleClasses), [DeviceClasses](#DeviceClasss), [DataTypes](#DataTypes) and [ProductClasses](#ProductClasses) in one referencable logical group.

#### Attributes
- **id** : The identifier for that *Domain*. Required.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *Domain*. Optional.
- **Imports** : XML import/include of other XML files / SDT templates. Optional.  
Note, that when using **Imports** one must include the following namespace in the Domain-Element definition, as shown in the example below: ```xmlns:xi="http://www.w3.org/2001/XInclude"```
- **[DataTypes](#DataType)** : A list of *DataTypes* that are global to the whole domain. They can be referenced from other definitions by the [Extend](#Extend) element. Optional.
- **[SubDevices](#SubDevice)** : a List of *SubDevice* components. They can be referenced from other SubDevice definitions by the [Extend](#Extend) element. Optional.
- **[ModuleClasses](#ModuleClass)** : A list of those *ModuleClass* components that are global to the whole domain. Optional.
- **[DeviceClasses](#DeviceClass)** : a List of *DeviceClass* components. Optional.
- **[ProductClasses](#ProductClass)** : a List of *ProductClass* components. Optional.

#### Example
```xml
<Domain xmlns:xi="http://www.w3.org/2001/XInclude"
    xmlns="http://www.onem2m.org/xml/sdt/4.0">
    id="org.onem2m.home.device">

    <Doc>Some documentation</Doc>
    <Imports>
      <!-- Import other SDTs via XInclude's include mechanism -->
        <xi:include href="anotherSDT.xml" parse="xml" />
    </Imports>
    <ModuleClasses>
      <!-- List of Domain global ModuleClasses goes here -->
    </ModuleClasses>
    <DeviceClasses>
      <!-- List of Devices goes here -->
  </DeviceClasses>
</Domain>
```

---

<a name="ModuleClass"/></a>
### ModuleClass
Element of [Domain](#Domain) and [Product](#Product).

![](images/ModuleClass.png)

*ModuleClass* elements are basically constraints or templates for how to model functionality of real things/appliances/devices within the [Domain](#Domain). *ModuleClasses* are the central element of the SDT to wrap functionalities.

A *ModuleClass* can extend another *ModuleClass* with additional functionalities.

Every [DeviceClass](#DeviceClass) can be described by a collection of *ModuleClasses* (functionality). 

The set of *ModuleClasses* is defined at the [Domain](#Domain) level or in the context of a [DeviceClass](#DeviceClass). 

In principle there could be an infinite number of *ModuleClasses*, for every kind of functionality found in automation protocol ... However that would not simplify the job of system architects and software developers at all. Therefore, oneM2M recommends that a finite and convenient number of prototypical *ModuleClasses* are re-used as much as possible (within a domain at least). oneM2M TS-0023 specification defines a rich set of such standardized *ModuleClasses* (as well as [DeviceClass](#DeviceClass) and [SubDevice](#SubDevice) models).

Typical *ModuleClasses* might be equivalent to "power ON/OFF", "Open/Close", "PanUP/DOWN", "Temperature", etc. Those examples make it apparent that various read/write usage of parameters, invoking of actions and waiting for events might be needed in the different *ModuleClasses*, and a guideline for those structures is explained below.


#### Attributes
- **name** : Name of the *ModuleClass*. The name must be unique in the scope of the [Domain](#Domain). Required.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.
- **minOccurs** : An optional string attribute that indicates the minimum cardinality of a *ModuleClass*, for example, in a [DeviceClass](#DeviceClass) definition. Optional. The default is "1".  
Note, if only the *minOccurs* attribute is specified, it can't be greater than 1, because the default value for *maxOccurs* is 1.
- **maxOccurs** : An optional string attribute that indicates the maximum cardinality of a *ModuleClass*, for example, in a [DeviceClass](#DeviceClass) definition. Optional. The default is "1".  
Note, if only the *maxOccurs* attribute is specified, it can't be smaller than 1, because the default value for *minOccurs* is 1.  
Also note, that if both *minOccurs* and *maxOccurs* are specified, then *maxOccurs* always must have an equal or higher cardinality than *minOccurs*.

#### Elements

- **[Doc](#Documentation)** : Documentation for the *ModuleClass*. Optional.
- **[Extend](#Extend)** : Reference to a another *ModuleClass* which is extended by this *ModuleClass*. Optional.
- **[Properties](#Property)** : Further meta-data (or properties) about the *ModuleClass*. Optional.
- **[Actions](#Action)** : A list of *Action* components, each defining a single action. Optional.
- **[Data](#DataPoint)** : A list of *DataPoint* components. Optional.
- **[Events](#Event)** : A list of *Event* components. Optional.

#### Example
```xml
<ModuleClass name="BooleanState">
  <Doc>Some documentation</Doc>
  <Actions>
    <!-- List of Actions goes here-->
  </Actions>
  <Events>
    <!-- List of Events goes here-->
  </Events>
  <Data>
    <!-- List of DataPoints goes here-->
  </Data>
</ModuleClass>
```

---

<a name="DataPoint"/></a>
### DataPoint
Element of [ModuleClass](#ModuleClass) and [Event](#Event).

![](images/DataPoint.png)

A *DataPoint* element represents an aspect of a device which can be read/written to, and forms part of a device’s data model. Manipulating *DataPoints* is the most common way of controlling devices. Each *DataPoint* has an associated *[DataType](#DataType)* (e.g. simple integer/real numbers, string of text, struct, or arrays thereof, or an enumeration) which facillitates data integrity. 

Note, that all RESTful systems (e.g. CoAP) use only *DataPoint* operations, so the mapping of a data models using an SDT into RESTful applications is easy.

However, *DataPoints* are not the only way of controlling and monitoring devices, so further [Actions](#Action) and [Events](#Event) are described below.

Though *DataPoints* only refer to single data points of a physical device it is possible to describe hierarchies by model the path to the data point in the hierarchy by a path-like structure like to the pathname of a UNIX file system. Here, the root node of the hierarchy is a slash (/ 0x2F) and the segments or nodes along the path are also separated by slashes. The actual datapoint is the last leaf at the path. 

In EBNF:

```ebnf
name          = dataPointName | "/" path ;  
path          = segment "/" path | dataPointName ;  
segment       = string ;  
dataPointName = string ;  
string        = (* character string excluding the character "/" *) ;
```

#### Attributes
- **name** : The name (and possible path in a hierarchical data model) of the *DataPoint*. The name must be unique in the scope of the [ModuleClass](#ModuleClass). Required.
- **optional**: Boolean that indicates whether a *DataPoint* is optional or mandatory. Optional, the default is *false*.
- **writable** : Boolean value that indicates whether this *DataPoint* is writable by an application. Optional. Default: true.
- **readable** : Boolean value that indicates whether this *DataPoint* is readable by an application. Optional. Default: true.
- **eventable** : Boolean value that indicates whether an internal or external change of this *DataPoint* raises an event. Optional. Default: false.
- **default** : A default value for the *DataPoint*.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *DataPoint*. Optional.
- **[DataType](#DataType)** : The type of the *DataPoint*. It must comply to the *DataType* definition. Required.

#### Example
```xml
<Data>
  <DataPoint  name="attributeName" writable="false">
    <Doc>Some documentation for the DataPoint</Doc>
    <DataType>
      <Simple type="string" />
    </DataType
  </DataPoint>
</Data>
```

---

<a name="Action"/></a>
### Action

Element of [ModuleClass](#ModuleClass).

![](images/Action.png)

*Action* elements are an efficient way of describing arbitrary sequences of operations/methods; these are very common in automation. Typical example include "FactoryReset", and "AutoCalibrate". *Actions* preserve transaction integrity by putting together all the parameters (see [Args](#Arg)) with the method which checks and executes them, in one step.

Note, that systems which rely on RESTful operations need to carry out such complex setup-parameters-then-do-action by first using (several) [DataPoint](#DataPoint) operations to "load" the parameters to the device and then do a [DataPoint](#DataPoint) operation to manipulate the "start operation NOW" action.

#### Attributes
- **name** : The name of the *Action*. The name must be unique in the scope of the [ModuleClass](#ModuleClass). Required.
- **optional**: Boolean that indicates whether an *Action* is optional or mandatory. Optional, the default is *false*.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *Action*. Optional.
- **[DataType](#DataType)** : The return type of the *Action*. It must comply to the *DataType* definition. Optional. If no *DataType* is specified the *Action* does not return a value.
- **[Args](#Arg)** : Zero or more occurances of argument definitions for an *Action*. Optional.

<a name="ActionExample"/></a>
#### Example
The following are two examples for actions implementing a getter and a setter for boolean values.
```xml
<Action name="get" type="boolean">
  <Doc>Obtain the current associated state. Example of a getter.</Doc>
</Action>

<Action name="setTarget">
  <Doc>Set the associated state to the specified value. Example of a setter.</Doc>
  <Args>
    <Arg name="value">
        <Doc>The desired value of the associated state.</Doc>
        <DataType>
          <Simple type="boolean" />
        </DataType>
      </Arg>
    </Args>
</Action>
```

---

<a name="Arg"/></a>
### Arg
Element of [Action](#Action).

![](images/Arg.png)
 
The *Arg* element represents the parameter information which a device needs to carry out a required *Action*. 

The *Arg* has the following attributes and elements:

#### Attributes
- **name** : The name of the *Arg* attribute. Required.
- **optional**: Boolean that indicates whether an *Arg* is optional or mandatory. Optional, the default is *false*.
- **default** : A default value for the *Arg*.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *argument*. Optional.
- **[DataType](#DataType)** : The return type of the *argument*. It must comply to the *DataType* definition. Required.

#### Example
See [the Action example above](#ActionExample).

---

<a name="Event"/></a>
### Event
Element of [ModuleClass](#ModuleClass).

![](images/Event.png)

*Event* elements are needed for automation protocols which "push" information, instead of relying on polling by the software application. A typical example would be a "SensorAlert" where a window sensor immediately transmits a change of its state from "closed" to "open", which could be used in a burglar alarm application, needs to be ready to accept such information immediately, and not wait for a regular polling of the device.

#### Attributes
- **name** : The name of the *Event*. The name must be unique in the scope of the [ModuleClass](#ModuleClass). Required.
- **optional**: Boolean that indicates whether an *Event* is optional or mandatory. Optional, the default is *false*.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *Event* Element. Optional.
- **[Data](#DataPoint)** : A list of *DataPoint* components for an event's payload. Optional.

#### Example
```xml
<Event name="stateChanged">
  <Doc>Some documentation for the Event</Doc>
  <Data>
    <DataPoint name="state">
      <DataType>
        <Simple type="boolean" />
      </DataType>
    </DataPoint>
  </Data>
</Event>
```
---

<a name="DeviceClass"/></a>
### DeviceClass
Element of [Domain](#Domain) and [Product](#Product).

![](images/DeviceClass.png)

The *DeviceClass* was initially thought of as the representation of "the basic things we are trying to model" and can still be considered so. However, after discussion with various SDOs, it was decided to add also "[sub-devices](#SubDevice)". That is, there is one level of hierarchy to allow modeling of e.g. a set of independent energy monitoring plugs in a single addressable power-extension-block. 

Note, that all the different devices which one needs to model within a domain are composed of one or more [ModuleClasses](#ModuleClass). 

For each physical device type on the network at least one *DeviceClass* **must** be defined. If the physical device is a simple device, i.e. it does not contain embedded devices, e.g. a light switch, it does not include further [SubDevices](#SubDevices). On the other hand, if the physical is a compound device, i.e. it does contain embedded devices that can be addressed separately, the *DeviceClass* **should** contain [SubDevices](SubDevices) for each of the identifiable embedded devices.

An example for a compound device  is a connected power-strip where each of the sockets can be switched on and off individually. The power-strip itself can provide functions such as "all sockets off" and "overall power consumption".

*DeviceClasses* may define their own [ModuleClasses](#ModuleClass) or refer to predefined [ModuleClasses](#ModuleClass) of the same or another [Domain](#Domain).

#### Attributes
- **id** : The identifier for that *DeviceClass*. The identifier must be unique at least in the scope of the domain, but the final scope is also influenced by implementing technologies. Required.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements

- **[Doc](#Documentation)** : Documentation for the *Device*. Optional.
- **[Properties](#Property)** : Further meta-data (or properties) about the *Device*. Optional.
- **[ModuleClasses](#ModuleClass)** : A list of *ModuleClass* components that are local to the *Device*. Optional.
- **[SubDevices](#SubDevice)** : A list of *SubDevice* components. Optional.

#### Example
```xml
<Device id="aDevice">
  <Doc>Some documentation</Doc>
  <Properties>
    <!-- The list of Properties for the Device goes here-->
  </Properties>
  <ModuleClasses>
    <!-- List of ModuleClasses local to the Device goes here-->
  </ModuleClasses>
  <SubDevices>
    <!-- List of Sub-Devices of the Device goes here-->
  </SubDevices>
</Device>
```

---

<a name="SubDevice"/></a>
### SubDevice
Element of [DeviceClass](#DeviceClass) and [Product](#Product).

*SubDevices* are optional components of a [DeviceClass](#DeviceClass). They represent physical sub-devices and services inside another device (the *DeviceClass*).

*SubDevices* may define their own [ModuleClasses](#ModuleClass) or extend *ModuleClasses* of its or another [Domain](#Domain).

![](images/SubDevice.png)

#### Attributes
- **id** : The identifier for that *SubDevice*. The identifier must be unique at least in the scope of the domain, but the final scope is also influenced by implementing technologies. Required.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.
- **minOccurs** : An optional string attribute that indicates the minimum cardinality of a *SubDevice*, for example, in a [DeviceClass](#DeviceClass) definition. Optional. The default is "1".  
Note, if only the *minOccurs* attribute is specified, it can't be greater than 1, because the default value for *maxOccurs* is 1.
- **maxOccurs** : An optional string attribute that indicates the maximum cardinality of a *SubDevice*, for example, in a [DeviceClass](#DeviceClass) definition. Optional. The default is "1".  
Note, if only the *maxOccurs* attribute is specified, it can't be smaller than 1, because the default value for *minOccurs* is 1.  
Also note, that if both *minOccurs* and *maxOccurs* are specified, then *maxOccurs* always must have an equal or higher cardinality than *minOccurs*.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *SubDevice*. Optional.
- **[Properties](#Property)** : Further meta-data (or properties) about the *SubDevice*. Optional.
- **[ModuleClasses](#ModuleClass)** : A list of *Module* components that are local to the *SubDevice*. Optional.

#### Example
```xml
<SubDevice id="aSubDevice">
  <Doc>Some documentation</Doc>
  <Properties>
    <!-- The list of Properties for the Device goes here-->
  </Properties>
  <ModuleClasses>
    <!-- List of ModuleClasses local to the Device goes here-->
  </ModuleClasses>
</SubDevice>
```

---
<a name="ProductClass"/></a>
### ProductClass
Element of [Domain](#Domain).

![](images/Product.png)

In real life of device manufacturing, there is the important concept of a *ProductClass* for a certain [DeviceClass](#DeviceClass). For example, oneM2M may specify a generic [DeviceClass](#DeviceClass) called *deviceSmartBracelt* with many features, using [ModuleClasses](#ModuleClass) to model the functionalities. Based on the same [DeviceClass](#DeviceClass), company A may design a *ProductClass* called "X-Fit" with only the [ModuleClass](#ModuleClass) of "stepCounter" and the instantiated [Property](#Property) value of "Manufacturer = Company A", while company B may design a *ProductClass* called "Y-Wristband" with the [ModuleClasseses](#ModuleClass) of "stepCounter" and "heartRateMonitor" and the instantiated [Property](#Property) value of "Manufacturer = Company B". Those two *ProductClasses* are different but follow the same [DeviceClass](#DeviceClass).

On the other hand, a *ProductClass* is **NOT** yet a real device instance of that [DeviceClass](#DeviceClass). It may not have an instantiated [Properties](#Property) like *device-ID*, *date-of-manufacturing*, and the firmware/software-version, etc. It can be ordered by the customers, but not necessarily instantiated or manufactured.

In short, a *ProductClass* is a concrete device model with deterministic [device Properties](#Property) and [ModuleClasses](#ModuleClass) without any optionality. It's deemed as an specialized implementation of a [DeviceClass](#DeviceClass) but not yet an device instance. Examples are the shopping items in an online digital store that can be ordered (but not necessarily manufactured) by a customer.

A *ProductClass* can be defined by implementing the functionalities of an existing [DeviceClass](#DeviceClass) while removing unimplemented optional [ Properties](#Property) and [ModuleClasses](#ModuleClass)), by extending from an existing [DeviceClass](#DeviceClass) and adding new [Properties](#Property) and [ModuleClasses](#ModuleClass), or from scratch not re-using on any [DeviceClass](#DeviceClass). 

#### Attributes
- **id** : The identifier for that *ProductClass*. The identifier must be unique at least in the scope of the domain, but the final scope is also influenced by the manufacturer or implementation domain. Required.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
All elements of [DeviceClass](#DeviceClass) can be reused in *ProductClass*, but the *optional* attribute of those elements is not applicable (ignored if present).

- **[Doc](#Documentation)** : Documentation for the *ProductClass*. Optional.
- **[Extend](#Extending)** : Extension of a parent *ProductClass* from which this *ProductClass* is extended. Optional.
- **[Properties](#Property)** : Further meta-data (or properties) about the *ProductClass*. Optional. 
- **[ModuleClasses](#ModuleClass)** : A list of *ModuleClass* components that are local to the *ProductClass*. Optional.
- **[SubDevices](#SubDevice)** : A list of *SubDevice* components. Optional.
- **ExtendDevice** : Extension of a [DeviceClass](#DeviceClass) which is implemented by this *ProductClass*. Optional.
  
**Note**: New extended *Properties* and *ModuleClasses* **shall** have different names from those in the extended *DeviceClass* if they're defined in the same *Domain*. Also, when extending a *ProductClass* that has a [DeviceClass](#DeviceClass) extended (or in any other parent in the inheritance hierarchy) then the child *ProductClass* **must not** extend another [DeviceClass](#DeviceClass).

#### Example
```xml
<ProductClass id="TestProductClass">
  <Doc>This is a new product</Doc>
  <Properties>
    <Property name="name" value="product-abc">
      <Simple type="string"></Simple>
    </Property>
    <Property name="vendor" value="xyz"/>
      <Simple type="string"></Simple>
    </Property>
    <Property name="SerialNumber"/>
      <Simple type="string"></Simple>
    </Property>
  </Properties>
  <ModuleClasses>
    <ModuleClass name="aModuleClass">
      <Extend domain="adomain" entity="aModuleClass" />
    </ModuleClass>
    <ModuleClass name="newModuleClass">
      <!-- List of Actions, Events and DataPoints goes here-->
    </ModuleClass>
  </ModuleClasses>
  <ExtendDevice>
    <Extend domain="adomain" entity="aDeviceClass" />
  </ExtendDevice>
</ProductClass>
```

---

<a name="Property"/></a>
### Property
Element of a [ProductClass](#ProductClass), [DeviceClass](#DeviceClass), [SubDevice](#SubDevice), and [ModuleClass](#ModuleClass).

![](images/Property.png)

*Property* elements are used to append to [ProductClasses](#ProductClass) [DevicesClasses](#DeviceClass) and their [ModuleClass](ModuleClass) elements with arbitrary additional information. For example, for a [ProductClass](#DeviceClass) it would be very common for a manufacturer to want to add into the XML file which is describing the manufacturer appliance such information as "Manufacturing Site", "Date of Manufacture", "Certification Code", "Energy Label Code", "compatible LAN technology", "URL for the device handbook", "physical limits of operation environments", etc.

Some of that information might in some devices be available by reading a specific device [DataPoint](#DataPoint), however even if it cannot be read from the device then at least it can be noted in the device's XML description. Examples for organizations that specify these kind of added "Property" information are [eCl@ss](http://www.eclass.eu) and [UNSPSC](http://www.unspsc.org) (United Nations Standard Products and Services Code).

Since the *Properties* are highly varied, depending on industry segment, no attempt is made in the SDT to constrain the options: however it is highly recommended to provide software-developer-friendly information in the [Doc](#Documentation) field of each Property. 

#### Attributes
- **name**: Name or identifier of a *Property*.
- **optional**: Boolean that indicates whether a *Property* is optional or mandatory. Optional, the default is *false*.
- **value**: Text representation of value of a *Property*. Optional.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *Property*. Optional.
- **DataType** : The data type of the property. This must be a [Simple](#Simple) type.

#### Example
```xml
<Property name="ManufacturedDate" value="2015.10.30 10:06">
  <Simple type="datetime" />
</Property>
```

---

<a name="DataTypes"/></a>
## Data Types


A data type can be simple integers or string text, or rather complex, as shown in the following diagram:

![](images/SDT_UML_DataType.png)

The various elements of *DataType* are described in the following sections.

*DataTypes* can be rather complex and it might be reasonable to define them on the [Domain](#Domain) level. This way the only need to be defined once and can be references through the [Extend](#Extend) element later, and also across [Domains](#Domain).


<a name="DataType"/></a>
### DataType

Element of a [Domain](#Domain), [Property](#Property), [Action](#Action), [Arg](#Arg), [DataPoint](#DataPoint), [Event](#Event), [Struct](#Struct), and [Array](#Array).

![](images/DataType.png)

The *DataType* element is a "container" for the various aspects of a type. 

#### Attributes
- **name** : The name of the *DataType*. Optional, with the exception that the name must be set for fields in a [Struct](#Struct) to distinguish individual fields in that structure.
- **unitOfMeasure** : Before considering the type of data in detail, there is the option to label the data with the units of measurement.  A "Temperature" measurement is meaningless until the units Kelvin, Celcius, Fahrenheit etc are known. Because of the extreme variety of units, a string field is the default annotation method, although of course a SDO could decide to reference a standardized list of units. Optional.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *DataType* Element. Optional.
- **TypeChoice** : This element is actual an element from the following list of data types:
  - **[Simple](#Simple)**
  - **[Struct](#Struct)**
  - **[Array](#Array)**
  - **[Enum](#Enum)**
- **[Constraint](#Constraint)** : A list of *Constraint* elements. Optional.

---

<a name="Constraint"/></a>
### Constraint
Element of [DataType](#DataType).

![](images/Constraint.png)

The *Constraint* element is an optional element allowing the manufacturer or organization to provide constraints on the permitted values of measured data or input parameters. It can significantly improve the reliability of software and validation of transmitted data.

#### Attributes
- **name** : The name or ID that identifies the *Constraint*. Required.
- **type** : The basic data type of the constraint. Note that this may be different from the type of the [DataType](#DataType) that this *Constraint* is assigned to. For example, a *Constraint* that specifies a maximum length of a string is of type *integer*. Optional.
- **value** : A pre-assigned value for the constraint, for example the maximum number of characters in a string. Optional.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *Constraint* Element. Optional.

---

### TypeChoice
Construct of [DataType](#DataType).
 
![](images/TypeChoice.png)

The *TypeChoice* construct is required for syntactic reasons in the UML diagram and the choice from the enumerated list simply designates the complexity of the following DataType. It is not directly instantiated or addressed.

---

<a name="Simple"/></a>
### Simple
Element of [DataType](#DataType).

![](images/SimpleType.png)

The *Simple* element is required in order for software to understand the format of the associated data, e.g. are the bytes an integer or real value? The selection choosen by HGI is based on practical experience to include some specific types which are slightly more complex:

1. the (technically redundant) options of *date* and *time* - to avoid problems which can arise interpreting a *datetime* value; 
2. *url* because it is expected to become extremely common to provide links to other data sources; 
3. the "blob" type to represent binary data of arbitrary structure.

If not stated otherwise datatypes should comply to the equivalent datatypes defined in [XML Schema Part 2: Datatypes Second Edition](http://www.w3.org/TR/xmlschema-2/#boolean):

![](images/BasicType.png)

- **boolean** : A boolean value as defined in [http://www.w3.org/TR/xmlschema-2/#boolean](http://www.w3.org/TR/xmlschema-2/#boolean) .
- **byte** : An integer datatype with the range of [0 - 255] as defined in [http://www.w3.org/TR/xmlschema-2/#unsignedByte](http://www.w3.org/TR/xmlschema-2/#unsignedByte) .
- **integer** : An integer value as defined in [http://www.w3.org/TR/xmlschema-2/#integer](http://www.w3.org/TR/xmlschema-2/#integer) .
- **float** : An IEEE single-precision 32-bit floating point type as defined in [http://www.w3.org/TR/xmlschema-2/#float](http://www.w3.org/TR/xmlschema-2/#float) .
- **string** : The string datatype represents character strings as defined in [http://www.w3.org/TR/xmlschema-2/#string](http://www.w3.org/TR/xmlschema-2/#string) .
- **date** : A date value as defined in [http://www.w3.org/TR/xmlschema-2/#date](http://www.w3.org/TR/xmlschema-2/#date) .
- **time** : A time value as defined in [http://www.w3.org/TR/xmlschema-2/#time](http://www.w3.org/TR/xmlschema-2/#time) .
- **datetime** : A time value as defined in [http://www.w3.org/TR/xmlschema-2/#dateTime](http://www.w3.org/TR/xmlschema-2/#dateTime) .
- **blob** : A blob value represents a binary object. The internal encoding is transparent and not defined here. The binary object must be encoded conforming to [http://www.w3.org/TR/xmlschema-2/#base64Binary](http://www.w3.org/TR/xmlschema-2/#base64Binary) .
- **uri** : A URI that represents a Uniform Resource Identifier Reference (URI) as defined by as defined in [RFC 2396](http://www.ietf.org/rfc/rfc2396.txt) and amended in [RFC 2732](http://www.ietf.org/rfc/rfc2732.txt) .
- **void** : The data type *void* represents the absence of a value. This data type can be used for [Actions](#Action) that don't return any value.


### Example
```xml
<DataPoint name="simpleDataPoint">
  <DataType>
    <Simple type="string" />
  </DataType>
</DataPoint>
```
---

<a name="Struct"/></a>
### Struct
Element of [DataType](#DataType).

![](images/Struct.png)

The *Struct* element can be used to represent an ordered list of diverse DataTypes, which are represented by the *name* attribute of each [DataType](#DataType), and can be used recursively. 

#### Elements
- **[DataType](#DataType)** : A list of *DataTypes* elements representing the elements of a structure.

### Example
```xml
<DataPoint name="structDataPoint">
  <DataType name="structured">
    <Struct>
      <DataType name="aString">
        <Simple type="string" />
      </DataType>
      <DataType name="anInteger">
        <Simple type="integer" />
      </DataType>
    </Struct>
  </DataType>
</DataPoint>
```

---

<a name="Array"/></a>
### Array 
Element of [DataType](#DataType).

![](images/Array.png)

The *Array* element is provided for defining lists of data; the definition is recursive so that multi-dimensional arrays can be described. Note that a Constraint can be used to provide limits on Array size.

#### Elements
- **[DataType](#DataType)** : A single *DataType* element that specifies the data type for the elements of the array.


### Example
```xml
<DataPoint name="arrayDataPoint">
  <DataType name="arrayOfInteger">
    <Array>
      <DataType>
        <Simple type="integer" />
      </DataType>
    </Array>
  </DataType>
</DataPoint>
```
---

<a name="Enum"/></a>
### Enum 
Element of [DataType](#DataType).

![](images/Enum.png)

The *Enum* element is provided for defining an enumeration data type; it defines a (possibly mixed) typed set of [EnumValues](#EnumValue).

#### Elements
- **[EnumValue](#EnumValue)** : A list of *EnumValue" element that specifies the data type for the elements of the array.

<a name="EnumExample"/></a>
#### Example
```xml
<DataPoint name="enumDataPoint">
  <DataType name="enumOfInteger">
    <Enum>
      <EnumValue name="red" value="1" type="integer" />
      <EnumValue name="green" value="2" />
      <EnumValue name="blue" value="3" />
    </Enum>
  </DataType>
</DataPoint>
```

---

<a name="EnumValue"/></a>
### EnumValue 
Element of [Enum](#Enum).

![](images/EnumValue.png)

The *Enum* element is provided for defining an enumeration data type; it defines a (possibly mixed) typed set of [EnumValues](#EnumValue).

#### Attributes
- **name** : The name that identifies the *EnumValue*. Required.
- **value** : An assigned value for the *EnumValue*. Required.
- **type** : The *BasicType* of the *EnumValue*. An [Enum](#Enum) may have *EnumValues* of mixed type. Optional, the default is *integer*.
- **semanticURI** : An attribute that contains a URI to a semantic description of the element. Optional.

#### Elements
- **[Doc](#Documentation)** : Documentation for the *EnumValue* Element. Optional.

#### Example
See [example in *Enum*](#EnumExample).

---

<a name="Extending"/></a>
## Extending (Inheriting)

The SDT supports inhering for certain element types. This is done by extending a parent element definition. 

By default, this method can be used in a simple way by just extending a whole parent element, but it is also possible to explicitly include or exclude certain elements from the derived element.

Extension across [Domains](#Domain) is supported and encouraged.

![](images/SDT_UML_Extend.png)

<a name="ExtendExample"/></a>
#### Example
The following example defines two [ModuleClasses](#ModuleClass), where the second definition extends to first one, but explicitly excludes one of the [DataPoints](#DataPoint).

```xml
<ModuleClasses>
  <ModuleClass name="aModuleClass">
    <Data>
      <DataPoint name="dataPoint1">
        <!-- type definition -->
      </DataPoint>
      <DataPoint name="dataPoint2">
        <!-- type definition -->
      </DataPoint>
    </Data>
  </ModuleClass>
  <ModuleClass name="anotherModuleClass">
    <Extend domain="aDomain" entity="aModuleClass">
      <Exclude name="dataPoint2" type="datapoint" />
    </Extend>
    <!-- more local definitions, such as Actions, DataPoints etc  -->
  </ModuleClass>
</ModuleClasses>
```

---

<a name="Extend"/></a>
### Extend
Element of [ProductClass](#ProductClass), [ModuleClass](#ModuleClass) and [DataType](#DataType).

![](images/Extend.png)

The *Extend* element specifies which element to extend. As a simple element it can be used to fully include the referenced element into the derived element, but it is also possible to explicitly include or exclude certain elements. 

Please note, that it is only possible to either [Include](#ExtendInclude) or [Exclude](#ExtendExclude) elements. When specifying *Include* elements, then all elements that should be included must be specified explicitly; all other elements are *not* included. When specifying *Exclude* elements, then all elements that should be excluded must be specified explicitly; all other elements are automatically included.

#### Attributes
- **domain** : Identifier / Reference of the [Domain](#Domain) of the extended element. Required.
- **entity** : Name of the element in the [Domain](#Domain) that is extended. Required.

#### Elements
- **ExtendChoice** : This element is actual a list of elements from the following list of extend choices. Please note, that it is only possible to either **Include** or **Exclude** elements.
  - **[Include](#ExtendInclude)**
  - **[Exclude](#ExtendExclude)**

#### Example
See [example in *Extending (Inheriting)*](#ExtendExample).

---

<a name="ExtendInclude"/></a>
### Include
Element of [Extend](#Extend).

![](images/ExtendInclude.png)

This element specifies a single element from the parent that will be included while extending.

#### Attributes
- **name** : The name that identifies the element to be included. Required.
- **type** : The [ExtendType](#ExtendType) of the element that is included. Optional, the default is *datapoint*.


---

<a name="ExtendExclude"/></a>
### Exclude
Element of [Extend](#Extend).

![](images/ExtendExclude.png)

This element specifies a single element from the parent that will be excluded while extending.

#### Attributes
- **name** : The name that identifies the element to be exclued. Required.
- **type** : The [ExtendType](#ExtendType) of the element that is excluded. Optional, the default is *datapoint*.

---

<a name="ExtendType"/></a>
### ExendType
Element of [ExtendInclude](#ExtendInclude) and [ExtendExclude](#ExtendExclude).

![](images/ExtendType.png)

This is an enumeration of element types for inclusion or exclusion while extending an element.

---

<a name="Documentation"/></a>
## Documentation (Doc)
Element for documentation in all elements.

![](images/Doc.png)

*Doc* elements (optional for all the above Elements) are very important to help understand the software-readable information for specific devices and services. They contain the human-readable information. Many automation protocols describe every possible operation in a comprehensive specification, however SDT is designed to include the relevant information at the "point of use" for the software developer, inside the SDT (and XML files based on it).

The text inside the *Doc* element can be structure using a very limited subset of HTML elements. The possible structuring is defined in EBNF as follows:

```ebnf
Doc            = "<Doc>" docContent "</Doc" ;
docContent     = docText | { paragraph | image } ;
docText        = { text | emphasizedText | boldText | monotypeText } ;
emphasizedText = "<em>" text "</em>" ;
boldText       = "<b>" text "</b>" ;
monotypeText   = "<tt>" text "</tt>" ;
paragraph      = "<p>" docText "</p>" ;
image          = "<img src=" url ">" "<caption>" text "</caption>" "</img>" ;
url            = "\"" (* valid URL *) "\"" ;
text           = (* XML text element *) ;
```

The intended use for each element is:

- **emphasizedText** : Emphasize the included text, e.g. printing it in italics font.
- **boldText** : Print the included text in bold font.
- **monotypeText** : Print the included text in a monospaced fonttype, e.g. to emphasize source code.
- **paragraph** : Structure the text in paragraphs.
- **image** : Include an image in the text. The image is loaded from the specified URL and must include a caption text.





