# JSON Serialization 

## Table of Contents

1. [Introduction](#introduction)
2. [SDT mapping to JSON](#mappings)
	1. [Elements](#elements)
	2. [Attributes](#attributes)
		1. [ExtendType](#ExtendType)
	3. [Imports](#imports)
	4. [Data Types](#dataTypes)
		1. [SimpleType](#SimpleType)
		2. [Array](#Array)
		3. [Struct](#Struct)
		4. [Enum](#Enum)
		5. [Extending DataTypes](#extendingDatatypes)



<a name="introduction"/></a>
## Introduction

When the Smart Device Template was introduced, XML was the favorite encoding format for schemas. Since then, JSON became more popular in developers' communities, especially in the context of RESTful implementations. 

JSON is more concise and human-readable comparing to XML, but has less capability of formal schema definition and validation (such as the XML Schema as defined in [domain.xsd](../src/domain.xsd)). 

For these reasons, JSON serialization is supported since SDT version 4.0.

There are some open tools such as [JSON Schema](http://json-schema.org/) that might be useful for describing and validating the JSON serialization of SDT, but it's not yet a formal standard and is still under development.

In this release, we chose to define the JSON serialization by the documentation below, while borrowing some valuable aspects from JSON Schema (e.g. data types and validation keywords). Full endorsement of [JSON Schema](http://json-schema.org/) may be considered in the future.

The general JSON structure follows the general SDT structure presented in [SDT_UML.uxf](SDT_UML.uxf) and described in [SDT_Components.md](SDT_Components.md).

<a name="mappings"/></a>
## SDT mapping to JSON

<a name="elements"/></a>
### Elements mapping

| SDT XML Elements                               | JSON Key Words | Remark                                                    |
|:-----------------------------------------------|:---------------|:----------------------------------------------------------|
| [Action](SDT_Components.md#Action)             | Action         |                                                           |
| Actions                                        | Actions        | This element is an array of *Action* structures.          |
| [Arg](SDT_Components.md#Arg)                   | Arg            |                                                           |
| Args                                           | Args           | This element is an array of *Arg* structures.             |
| Constraint                                     | Constraint     |                                                           |
| Constraints                                    | Constraints    | This element is an array of *Constraint* structures.      |
| Data                                           | Data           | This element is an array of *DataPoint* structures.       |
| [DataPoint](SDT_Components.md#DataPoint)       | DataPoint      |                                                           |
| [DataType](SDT_Components.md#DataType)         | DataType       |                                                           |
| [DataTypes](SDT_Components.md#DataTypes)       | DataTypes      | This element is an array of *DataType* structures.        |
| [DeviceClass](SDT_Components.md#DeviceClass)   | DeviceClass    |                                                           |
| DeviceClasses                                  | DeviceClasses  | This element is an array of *DeviceClass* structures.     |
| [Doc](SDT_Components.md#Documentation)         | Doc            |                                                           |
| [Domain](SDT_Components.md#Domain)             | Domain         |                                                           |
| [Event](SDT_Components.md#Event)               | Event          |                                                           |
| Events                                         | Events         | This element is an array of *Event* structures.           |
| Exclude                                        | Exclude        |                                                           |
| Excludes                                       | Excludes       | This element is an array of *Exclude* structures.         |
| [Extend](SDT_Components.md#Extending)          | Extend         |                                                           |
| Imports                                        | Imports        | This is an array of URLs or file URLs to files to import. |
| Include                                        | Include        |                                                           |
| Includes                                       | Includes       | This element is an array of *Include* structures.         |
| [ModuleClass](SDT_Components.md#ModuleClass)   | ModuleClass    |                                                           |
| ModuleClasses                                  | ModuleClasses  | This element is an array of *ModuleClass* structures.     |
| [ProductClass](SDT_Components.md#ProductClass) | ProductClass   |                                                           |
| ProductClasses                                 | ProductClasses | This element is an array of *ProductClasses* structures.  |
| Properties                                     | Properties     | This element is an array of *Property* structures.        |
| [Property](SDT_Components.md#Property)         | Property       |                                                           |
| [SubDevice](SDT_Components.md#SubDevice)       | SubDevice      |                                                           |
| SubDevices                                     | SubDevices     | This element is an array of *SubDevice* structures.       |

#### Example

The following example defines a simple *Light* device.

**XML**:

```xml
<Domain xmlns="http://www.onem2m.org/xml/sdt/4.0" id="SimpleExample" >
	<DeviceClasses>
		<DeviceClass id="Light">
			<Doc>This is a very simple device representing a light.</Doc>
			<ModuleClasses>
				<ModuleClass name="Switch">
					<Data>
						<DataPoint name="status" readable="true" writable="true">
							<Doc>This property indicates the ON/OFF status.</Doc>
							<DataType>
								<SimpleType type="boolean" />
							</DataType>
						</DataPoint>
					</Data>
				</ModuleClass>
			</ModuleClasses>
		</DeviceClass>
	</DeviceClasses>
</Domain>
```

**JSON**:

```javascript
{
	"Domain": {
		"id": "SimpleExample",
		"DeviceClasses": [
			{
				"DeviceClass": {
					"id": "Light",
					"Doc": "This is a very simple device representing a light.",
					"ModuleClasses": [
						{
							"ModuleClass": {
								"Data": [
									{
										"DataPoint": {
											"name": "status",
											"readable": "true",
											"writable": "true",
											"Doc": "This property indicates the ON/OFF status.",
											"DataType": "boolean"
										}
									}
								]
							}
						}
					]
				}
			}
		]
	}
}
```

<a name="attributes"/></a>
### Attributes mapping
There are common attributes (e.g. *@id, @name, @optional*) used in several SDT components (e.g. in *DeviceClass, ModuleClass*) as well as attributes used only in certain components (e.g. *@entity* in *Extend*). XML attributes are represented as normal fields in JSON. The mapping of those attributes to JSON follows the rules as below. 

| SDT XML Attributes | JSON Key Word |
|:-------------------|:--------------|
| @default           | default       |
| @domain            | domain        |
| @entity            | entity        |
| @eventable         | eventable     |
| @href              | href          |
| @id                | id            |
| @maxOccurs         | maxOccurs     |
| @minOccurs         | minOccurs     |
| @name              | name          |
| @optional          | optional      |
| @parse             | parse         |
| @readable          | readable      |
| @semanticURI       | semanticURI   |
| @type              | type          |
| @unitOfMeasure     | unitOfMeasure |
| @value             | value         |
| @writable          | writable      |


<a name="imports"/></a>
### Import mapping

The imports are mapped to an array of *Include* structures. Each *Include* structure contains a *href* and a *parse* element.  
A parser must make sure to import and regard the definitions in the referenced files.

#### Example

**XML**:

```xml
<Imports>
	<xi:include href="anotherSDT.xml" parse="xml" />
</Imports>
```


**JSON**:

```javascript
"Imports" : [
	{ "Include" : { "href" : "anotherSDT.xml", "parse" : "json" } }
]
```

<a name="ExtendType"/></a>
### ExtendType mapping
In the *Include* and *Exclude* elements one may specify the type of the element to be included or excluded. The following table shows the mapping of [ExtendType](SDT_Components.md#ExtendType).

| SDT ExtendType | JSON Key Word |
|:---------------|:--------------|
| action         | action        |
| datapoint      | datapoint     |
| event          | event         |
| moduleclass    | moduleclass   |
| property       | property      |
| device         | device        |
| subdevice      | subdevice     |


<a name="dataTypes"/></a>
### Data Type mapping
Data types are defined in [SDT_Components.md](SDT_Components.md).

The mapping of the different kind of data types to JSON happens directly to JSON structures. The identifier for the data type kind is the name of that structure. This means that the data type definition is not wrapped by a *DataType* structure.


<a name="SimpleType"/></a>
### SimpleType

Note for [SimpelType](SDT_Components.md#SimpelType): For simplicity, if there is no other attribute present then the type value of [SimpelType](SDT_Components.md#SimpelType) can be directly put as the value of parent [DataType](SDT_Components.md#DataType) key, e.g. ```"DataType":"string"```

| SDT Data Types | Note                                                                                                |
|:---------------|:----------------------------------------------------------------------------------------------------|
| boolean        | boolean (JSON Schema)                                                                               |
| string         | string (JSON Schema)                                                                                |
| byte           | An integer datatype with the range of [0 - 255]                                                     |
| integer        | integer (JSON Schema)                                                                               |
| float          | number (JSON Schema)                                                                                |
| date           | date (JSON Schema)                                                                                  |
| time           | time (JSON Schema)                                                                                  |
| datetime       | date-time  (JSON Schema)                                                                            |
| blob           | A *base64Binary* encoded string according to [RFC 2045](https://www.w3.org/TR/xmlschema-2/#RFC2045) |
| uri            | A string following the URI format (JSON Schema)                                                     |
| void           | null (JSON Schema)                                                                                  |

#### Examples

Simple data type:

**XML**:

```xml
<DataType>
	<SimpleType type="string"/>
</DataType>
```

**JSON**:

```javascript
"SimpleType" : "string"
```

A simple data type with additional attributes:

**XML**:

```xml
<DataType name="temperatureType" unitOfMeasure="C">
	<SimpleType type="float"/>
</DataType>
```

**JSON**:

```javascript
"SimpleType" : { 
	"name" : "temperatureType",
	"unitOfMeasure" : "C",
	"type" : "number"
}
```

<a name="Array"/></a>
### Array

An [Array](SDT_Components.md#Array) definition is mapped to an *Array* JSON structure. The content of the structure is only one element that defines the type of the array. This could be any [SimpleType](#SimpleType), [Array](#Array), [Struct](#Struct), or [Enum](#Enum).

#### Example

A simple data type with additional attributes:

**XML**:

```xml
<DataType>
	<Array>
		<DataType>
			<SimpleType type="float" />
		</DataType>
	</Array>
</DataType>
```

**JSON**:

```javascript
"Array" : { 
	"SimpleType" : "float"
}
```

<a name="Struct"/></a>
### Struct

A [Struct](SDT_Components.md#Struct) definition is mapped to a *Struct* JSON array. The content of the array are the individual data types for the structure. These could be any [SimpleType](#SimpleType), [Array](#Array), [Struct](#Struct), or [Enum](#Enum).

A *Struct* definition must contain a *name* attribute.

#### Example

A simple structured data type (a structure that contains an *indentifier* variable of type *string* and a *count* variable of type *integer*).

**XML**:

```xml
<DataType>
	<Struct>
		<DataType name="identifier">
			<SimpleType type="string" />
		</DataType>
		<DataType name="count">
			<SimpleType type="integer" />
		</DataType>
	</Struct>
</DataType>
```

**JSON**:

```javascript
"Struct": [
	{
		"SimpleType": {
			"name": "identifier",
			"type": "string"
		},
		"SimpleType": {
			"name": "count",
			"type": "integer"
		}
	}
]
```

A more complex structured data type (a structure that contains an indentifier variable of type *string* and an *integer* array named *items*).


**XML**:

```xml
<DataType>
	<Struct>
		<DataType name="identifier">
			<SimpleType type="string" />
		</DataType>
		<DataType name="items">
			<Array>
				<DataType>
					<SimpleType type="integer" />
				</DataType>
			</Array>
		</DataType>
	</Struct>
</DataType>
```

**JSON**:

```javascript
"Struct": [
	{
		"SimpleType": {
			"name": "identifier",
			"type": "string"
		},
		"Array": {
			"name": "items",
			"SimpleType" : "float"
		}
	}
]
```

<a name="Enum"/></a>
### Enum

A [Enum](SDT_Components.md#Enum) definition is mapped to an *Enum* JSON structure. The content of the structure are the individual enum values of the enum. 

A *Enum* definition must contain a *name* attribute.

#### Example

An *Enum* that defines three [EnumValues](SDT_Components.md#EnumValues). The first definition explicitly specifies the type of the [EnumValues](SDT_Components.md#EnumValues).

**XML**:

```xml
<DataType>
	<Enum>
		<EnumValue name="red" value="1" type="integer"/>
		<EnumValue name="green" value="2" />
		<EnumValue name="blue" value="3" />
	</Enum>
</DataType>
```

**JSON**:

```javascript
"Enum": [
	{ "EnumValue" : { "name" : "red", "value" : 1, "type" : "integer" } },
	{ "EnumValue" : { "name" : "green", "value" : 2 } },
	{ "EnumValue" : { "name" : "blue", "value" : 3 } }
]
```

<a name="extendingDatatypes"/></a>
### Extending Data Types

Extending [DataTypes](SDT_Components.md#DataTypes) are mapped the same way as extending, for example, [ModuleClasses](SDT_Components.md#ModuleClass). However, since the [DataType](SDT_Components.md#DataTypes) is open, the generic form [DataType](SDT_Components.md#DataTypes) must be used for JSON serialization. It maps to a structure that contains a single [Extend](SDT_Components.md#Extending) structure. This structure contains the two attributes *domain* and *entity*.

#### Example

An [Enum](SDT_Components.md#Enum) that defines three *EnumValues*. The first definition explicitly specifies the type of the *EnumValue*.

**XML**:

```xml
<DataType>
  <Extend domain="aDomain" entity="temperatureType" />
</DataType>
```

**JSON**:

```javascript
"DataType": {
	"Extend": { "domain": "aDomain", "entity": "temperatureType" }
}
```